using UnityEngine;
using System;
using UnityEngine.EventSystems;
using System.Collections.Generic;
using UnityEngine.UI;
using System.Linq;
using System.Text;
using MVR.FileManagementSecure;

namespace Ruvik
{
    class PHHands
    {
        static public JSONStorable atomPlugin;
        static public GenerateDAZMorphsControlUI activeAtomsMorphController;
        public static List<JointSelector> activeJoints = new List<JointSelector>();
        public static List<string> usedAtoms = new List<string>();
        public static bool canPoseHands, isAdvancedMode = false;
        public static Dictionary<string, List<string>> retrivedJoints = new Dictionary<string, List<string>>();
        public static bool warned = false;
        public static Dictionary<string,string> Retrivedside= new Dictionary<string, string>();
        private static StringBuilder outputBuilder = new StringBuilder();
        public static string presetsPath = SuperController.singleton.savesDir + "HandPose";
        public class RetrivedJoint
        {
            public string atomName;
            public List<string> joints;
            public RetrivedJoint(string name,List<string> joints)
            {
                atomName = name;
                this.joints = joints;
            }
        }
        public static void UpdateRetrivedSides()
        {
            if (PHUtils.activeAtom == null) return;
            Retrivedside[PHUtils.activeAtom.name] = PHUI.isRightSide.ToString();
        }
        public static bool ActiveAtomHasPlugin()
        {
            return PHUtils.activeAtom.GetStorableIDs().Any((storable) => storable.Contains("PHFingerSliders"));
        }
        public static void LoadPose(string path)
        {
            if (string.IsNullOrEmpty(path)) return;
            PHUI.phHandPosingAdvanced.SetActive(true);
            PHUI.phHandPosingMinimal.SetActive(true);
            bool warn = false;
            bool canLoad =  ActiveAtomHasPlugin() && usedAtoms.Contains(PHUtils.activeAtom.uid);
            bool oldFile = true;
            foreach (var slider in GameObject.FindObjectsOfType<PHHands.PHSlider>())
            {
                slider.ResetValue();
            }
            if (canLoad)
                foreach (var joint in GameObject.FindObjectsOfType<PHHands.JointSelector>())
                {
                    joint.joints[PHUI.isRightSide ? 0 : 1].Reset();
                }
            if (path.EndsWith(".jpg"))
                path = path.Substring(0, path.Length - "jpg".Length) + "json";
            //read file
            string LoadedHandPose = SuperController.singleton.ReadFileIntoString(path);
            string[] LoadedHandPoseLines = LoadedHandPose.Split(new string[] { "\r\n", "\n" }, StringSplitOptions.None);
            Vector3 sidemulti = PHUI.isRightSide ? new Vector3(1.0f, 1.0f, 1.0f) : new Vector3(1.0f, -1.0f, -1.0f);
            int handSide = PHUI.isRightSide ? 0 : 1;
            foreach (string s in LoadedHandPoseLines)
            {
                if (s.StartsWith("#"))
                    continue;
                if (s.Length == 0)
                    continue;
                if (s.StartsWith("MIN_"))
                {
                    oldFile = false;
                    float sliderValue = float.Parse(s.Split(new string[] { ":" }, StringSplitOptions.None)[1]);
                    string morphName = s.Split(':')[0].Split('_')[1];
                    activeAtomsMorphController.GetMorphByUid(PHUI.GetHandSideText() + " " + morphName).morphValue = sliderValue;
                }
                else
                {
                    if (!canLoad)
                    {
                        warn = true;
                        continue;
                    }
                    string[] segValues = s.Split(new string[] { ":" }, StringSplitOptions.None);
                    string[] segmentID = segValues[0].Split(new string[] { " " }, StringSplitOptions.None);
                    string[] values = segValues[1].Split(new string[] { " " }, StringSplitOptions.None);
                    if (segmentID[0] == "Middle")
                        segmentID[0] = "Mid";

                    JointSelector js = GameObject.Find($"PH_node_{segmentID[0]}{segmentID[1]}").GetComponent<JointSelector>();
                    js.joints[handSide].SetRotation(new Vector3(float.Parse(values[0]) * sidemulti.x, float.Parse(values[1]) * sidemulti.y, float.Parse(values[2]) * sidemulti.z));
                }
            }
            PHUI.phHandPosingAdvanced.SetActive(isAdvancedMode);
            PHUI.phHandPosingMinimal.SetActive(!isAdvancedMode);
            if (oldFile)
                GameObject.FindObjectsOfType<PHSlider>().ToList().ForEach(sl => sl.ResetValue());
            if (warn)
                SuperController.LogMessage("warning: can't load advanced hand pose without attaching PHFinger to the selected atom");
        }
        public static void SavePose()
        {
            if (!FileManagerSecure.DirectoryExists(presetsPath))
            {
                FileManagerSecure.CreateDirectory(presetsPath);
            }
            SuperController.singleton.GetMediaPathDialog((string path) =>
            {
                SavePose(path);
            }, "json", presetsPath, false, false, true, null, true);
            uFileBrowser.FileBrowser browser = SuperController.singleton.mediaFileBrowserUI;
            browser.SetTextEntry(true);
            browser.fileEntryField.text = ((int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds).ToString();
            browser.ActivateFileNameField();
        }
        public static void SavePose(string path)
        {
            if (path.Length == 0) return;
                outputBuilder.Length = 0;
            PHUI.phHandPosingAdvanced.SetActive(true);
            PHUI.phHandPosingMinimal.SetActive(true);
            if (ActiveAtomHasPlugin() && usedAtoms.Contains(PHUtils.activeAtom.uid))
            {
                if (PHUI.isRightSide)
                {
                    foreach (var js in GameObject.FindObjectsOfType<JointSelector>())
                    {
                        if (js.joints[0].joint.baseJointRotation == Vector3.zero)
                            continue;
                        if (js.jointID == "Middle")
                            js.jointID = "Mid";
                        outputBuilder.AppendLine(js.jointID + " " + js.jointNumber + ":" + js.joints[0].joint.baseJointRotation.x + " " + js.joints[0].joint.baseJointRotation.y + " " + js.joints[0].joint.baseJointRotation.z);
                    }
                }
                else
                {
                    foreach (var js in GameObject.FindObjectsOfType<JointSelector>())
                    {
                        if (js.joints[1].joint.baseJointRotation == Vector3.zero)
                            continue;
                        if (js.jointID == "Middle")
                            js.jointID = "Mid";
                        outputBuilder.AppendLine(js.jointID + " " + js.jointNumber + ":" + js.joints[1].joint.baseJointRotation.x + " " + -js.joints[1].joint.baseJointRotation.y + " " + -js.joints[1].joint.baseJointRotation.z);
                    }
                }
            }
            foreach (var slider in GameObject.FindObjectsOfType<PHSlider>())
            {
                outputBuilder.AppendLine("MIN_" + slider.morphName + ":" + slider.slider.value);
            }
            PHUI.phHandPosingAdvanced.SetActive(isAdvancedMode);
            PHUI.phHandPosingMinimal.SetActive(!isAdvancedMode);
            if (!path.EndsWith(".json"))
            {
                path += ".json";
            }

            SuperController.singleton.DoSaveScreenshot(path.Substring(0, path.Length - "json".Length) + "jpg");
            FileManagerSecure.WriteAllText(path, outputBuilder.ToString());
        }
        public static void SwitchMode()
        {
            isAdvancedMode = !isAdvancedMode;
            PHUI.phHandPosingAdvanced.SetActive(true);
            PHUI.phHandPosingMinimal.SetActive(true);
            if (isAdvancedMode && !ActiveAtomHasPlugin())
            {
                SuperController.LogMessage("to pose fingers in advance mode you need to add the \"PHFingerSlider.cs\" to the atom");
                isAdvancedMode = false;
            }
            PHUI.phHandPosingAdvanced.SetActive(isAdvancedMode);
            PHUI.phHandPosingMinimal.SetActive(!isAdvancedMode);
            PHUI.handModeText.text = isAdvancedMode ? "adv" : "min";
            UpdateSelectors();
        }
        public static void UpdateSliders()
        {
            UpdateSliders("thumb");
            UpdateSliders("index");
            UpdateSliders("mid");
            UpdateSliders("ring");
            UpdateSliders("pinky");
            UpdateSliders("in-Out", "fingers");
            UpdateSliders("in-Out", "thumb");
            UpdateSliders("fist", "hand");
            UpdateSliders("fist", "fingers");
            UpdateSliders("fist", "thumb");
            UpdateSliders("grasp", "hand");
            UpdateSliders("grasp", "fingers");
            UpdateSliders("grasp", "thumb");
            UpdateSliders("straighten", "hand");
            UpdateSliders("straighten", "fingers");
            UpdateSliders("straighten", "thumb");
            UpdateSliders("chop", "hand");
            UpdateSliders("spread", "hand");
        }
        public static void UpdateSelectors()
        {
            PHUI.phHandPosingMenu.SetActive(true);

            if (isAdvancedMode && !ActiveAtomHasPlugin())
                SwitchMode();

            if (ActiveAtomHasPlugin())
            {

                PHUI.phHandPosingAdvanced.SetActive(true);
                atomPlugin = PHUtils.activeAtom.GetStorableByID(PHUtils.activeAtom.GetStorableIDs().Find((storable) => storable.Contains("PHFingerSliders")));

                ClearActiveJoints();

                if (PHUtils.activeAtom.on)
                {
                    UpdateSelector("Thumb", 0);
                    UpdateSelector("Carpal", 0);
                    UpdateSelector("Index", 1);
                    UpdateSelector("Mid", 1);
                    UpdateSelector("Ring", 2);
                    UpdateSelector("Pinky", 2);
                    UpdateSelector("Thumb", 0);
                }

                if (!usedAtoms.Contains(PHUtils.activeAtom.uid))
                    usedAtoms.Add(PHUtils.activeAtom.uid);

                RetriveActiveJoints();
            }


            if (Retrivedside.ContainsKey(PHUtils.activeAtom.uid))
                if (Retrivedside[PHUtils.activeAtom.uid] != PHUI.isRightSide.ToString())
                {
                    PHUI.SwitchSides();
                    Retrivedside[PHUtils.activeAtom.uid] = PHUI.isRightSide.ToString();
                }

            PHUI.phHandPosingAdvanced.SetActive(isAdvancedMode);
            PHUI.phHandPosingMinimal.SetActive(true);
            activeAtomsMorphController = ((PHUtils.activeAtom.GetStorableByID("geometry")) as DAZCharacterSelector).morphsControlUI;
            UpdateSliders();
            PHUI.phHandPosingMinimal.SetActive(!isAdvancedMode);

            PHUI.phHandPosingMenu.SetActive(!PHUI.isBodyPosing);
        }
        public static void UpdateRetrivedJoints()
        {
            if (PHUtils.activeAtom == null) return;
            retrivedJoints[PHUtils.activeAtom.name + PHUI.isRightSide.ToString()] = activeJoints.Select(js => js.jointID + js.jointNumber).ToList();
        }
        static void RetriveActiveJoints()
        {
            if (retrivedJoints.ContainsKey(PHUtils.activeAtom.name + PHUI.isRightSide.ToString()))
            {
                foreach (var item in retrivedJoints[PHUtils.activeAtom.name + PHUI.isRightSide.ToString()])
                {
                    var js = GetJointSelectorByName(item);
                    if (!activeJoints.Contains(js))
                    {
                        activeJoints.Add(js);
                        js.tex.color = Color.green;
                    }
                }

            }
        }
        static public void ResetHandAdvanced()
        {
            if (ActiveAtomHasPlugin() && usedAtoms.Contains(PHUtils.activeAtom.name))
            {
                PHUI.phHandPosingAdvanced.SetActive(true);
                GameObject.FindObjectsOfType<JointSelector>().ToList().ForEach(selector => selector.joints[PHUI.isRightSide ? 0 : 1].Reset());
                PHUI.phHandPosingAdvanced.SetActive(isAdvancedMode);
            }
        }
        static public void ResetHandMinimal()
        {
            PHUI.phHandPosingMinimal.SetActive(true);
            GameObject.FindObjectsOfType<PHSlider>().ToList().ForEach(slider => slider.ResetValue());
            PHUI.phHandPosingMinimal.SetActive(!isAdvancedMode);
        }
        public static JointSelector GetJointSelectorByName(string name)
        {
            return GameObject.Find($"PH_node_{name}").GetComponent<JointSelector>();
        }
        private static void InitFingerSelector(string name)
        {
            int count = 3;
            if (name == "Carpal")
                count = 2;
            for (int i = 1; i <= count; i++)
            {
                GameObject.Find($"PH_node_{name}{i}").AddComponent<JointSelector>().SetName();
            }
        }
        public static void InitFingerSelectors()
        {
            InitFingerSelector("Thumb");
            InitFingerSelector("Carpal");
            InitFingerSelector("Index");
            InitFingerSelector("Mid");
            InitFingerSelector("Ring");
            InitFingerSelector("Pinky");
        }

        public static void SwitchSides()
        {
            ClearActiveJoints();
            PHUI.SwitchSides();
            Retrivedside[PHUtils.activeAtom.name] = PHUI.isRightSide.ToString();
            RetriveActiveJoints();
        }
        static string FirstLetterUpper(string str)
        {
            return char.ToUpper(str[0]) + str.Substring(1);
        }
        public static void UpdateSliders(string finger)
        {
            var sliderComp = GameObject.Find($"PH_fingers_{finger}_bend").GetComponent<PHSlider>();
            if (sliderComp == null)
                sliderComp = GameObject.Find($"PH_fingers_{finger}_bend").AddComponent<PHSlider>();
            if (finger == "thumb")
                sliderComp.SetSlider(FirstLetterUpper(finger) + " Bend");
            else
                sliderComp.SetSlider(FirstLetterUpper(finger) + " Finger Bend");
        }
        public static void UpdateSliders(string action, string target)
        {
            var sliderComp = GameObject.Find($"PH_{target}_{action}").GetComponent<PHSlider>();
            if (sliderComp == null)
                sliderComp = GameObject.Find($"PH_{target}_{action}").AddComponent<PHSlider>();
            sliderComp.SetSlider(FirstLetterUpper(target) + " " + FirstLetterUpper(action));
        }
        public static void UpdateSelector(string name, int carpal)
        {
            int count = 3;
            if (name == "Carpal")
                count = 2;
            for (int i = 1; i <= count; i++)
            {
                GameObject.Find($"PH_node_{name}{i}").GetComponent<JointSelector>().UpdateName(carpal);
            }
        }

        public static void ClearActiveJoints()
        {
            foreach (var joint in activeJoints)
            {
                joint.Clear();
            }
            activeJoints.Clear();

        }
        public class FingerJoint
        {
            public DAZBone joint;
            public JSONStorableFloat[] sliders;
            public FingerJoint(char side, string fingerName, int carpal)
            {
                var gender = PHUtils.activeAtom.transform.Find("rescale2/PhysicsModel/Genesis2Male") != null ? "Male" : "Female";
                sliders = new JSONStorableFloat[3];
                if (fingerName.Contains("Carpal"))
                {
                    joint = PHUtils.activeAtom.transform.Find($"rescale2/PhysicsModel/Genesis2{gender}/hip/abdomen/abdomen2/chest/{side}Collar/{side}Shldr/{side}ForeArm/{side}Hand/{side}{fingerName}").GetComponent<DAZBone>();
                }
                else
                {

                    int index = (int)fingerName[fingerName.Length - 1] - 48;
                    string NewfingerName = fingerName.Substring(0, fingerName.Length - 1);
                    string name = $"hip/abdomen/abdomen2/chest/{side}Collar/{side}Shldr/{side}ForeArm/{side}Hand/";

                    if (carpal > 0) name += $"{side}Carpal{carpal}/";

                    for (int i = 1; i <= index; i++)
                    {
                        name += $"{side}{NewfingerName}{i}/";
                    }
                    joint = PHUtils.activeAtom.transform.Find($"rescale2/PhysicsModel/Genesis2{gender}/{name}").GetComponent<DAZBone>();
                }

                sliders[0] = atomPlugin.GetFloatJSONParam($"{side}{fingerName}X"); sliders[0].valNoCallback = joint.baseJointRotation.x;
                sliders[1] = atomPlugin.GetFloatJSONParam($"{side}{fingerName}Y"); sliders[1].valNoCallback = joint.baseJointRotation.y;
                sliders[2] = atomPlugin.GetFloatJSONParam($"{side}{fingerName}Z"); sliders[2].valNoCallback = joint.baseJointRotation.z;

                if (!usedAtoms.Contains(PHUtils.activeAtom.uid))
                {
                    sliders[0].slider.onValueChanged.AddListener((f) => SetRotationX(f));
                    sliders[1].slider.onValueChanged.AddListener((f) => SetRotationY(f));
                    sliders[2].slider.onValueChanged.AddListener((f) => SetRotationZ(f));
                }
            }
            public void SetRotation(Vector3 rot)
            {
                sliders[0].val = rot.x;
                sliders[1].val = rot.y;
                sliders[2].val = rot.z;
            }
            public void SetRotationX(float f)
            {
                joint.baseJointRotation = new Vector3(f, joint.baseJointRotation.y, joint.baseJointRotation.z);
            }
            public void SetRotationY(float f)
            {
                joint.baseJointRotation = new Vector3(joint.baseJointRotation.x, f, joint.baseJointRotation.z);

            }
            public void SetRotationZ(float f)
            {
                joint.baseJointRotation = new Vector3(joint.baseJointRotation.x, joint.baseJointRotation.y, f);

            }
            public void Rotate(Vector3 rot)
            {
                sliders[0].val = joint.baseJointRotation.x > 120.0f ? 120.0f : joint.baseJointRotation.x < -120.0f ? -120.0f : joint.baseJointRotation.x + rot.x;
                sliders[1].val = joint.baseJointRotation.y > 120.0f ? 120.0f : joint.baseJointRotation.y < -120.0f ? -120.0f : joint.baseJointRotation.y + rot.y;
                sliders[2].val = joint.baseJointRotation.z > 120.0f ? 120.0f : joint.baseJointRotation.z < -120.0f ? -120.0f : joint.baseJointRotation.z + rot.z;
            }
            public void Reset()
            {
                joint.baseJointRotation = Vector3.zero;
                sliders[0].val = 0.0f;
                sliders[1].val = 0.0f;
                sliders[2].val = 0.0f;
            }
        }
        public class PHSlider : MonoBehaviour, IPointerEnterHandler, IPointerExitHandler
        {
            bool isMoused = false;
            public Slider slider;
            Image handle;
            public string morphName;
            public void ResetValue()
            {
                if (PHUI.isSymmetryToggled)
                {
                    activeAtomsMorphController.GetMorphByUid("Right" + " " + morphName).morphValue = 0.0f;
                    activeAtomsMorphController.GetMorphByUid("Left" + " " + morphName).morphValue = 0.0f;
                }
                else
                {
                    slider.value = 0.0f;
                    activeAtomsMorphController.GetMorphByUid(PHUI.GetHandSideText() + " " + morphName).morphValue = 0.0f;
                }
                UpdateSliderValue();
            }
            public void UpdateSliderValue()
            {
                slider.value = activeAtomsMorphController.GetMorphByUid(PHUI.GetHandSideText() + " " + morphName).morphValue;
            }
            public void SetSlider(string morphName)
            {
                this.morphName = morphName;
                this.slider = this.gameObject.GetComponent<Slider>();
                this.handle = this.gameObject.transform.Find("Handle Slide Area/Handle").GetComponent<Image>();
                UpdateSliderValue();
            }
            public void OnPointerEnter(PointerEventData eventData)
            {
                isMoused = true;
            }
            public void OnPointerExit(PointerEventData eventData)
            {
                isMoused = false;
            }
            public void Mirror()
            {
                if (PHUI.GetHandSideText() == "Right")
                    activeAtomsMorphController.GetMorphByUid("Left" + " " + morphName).morphValue = activeAtomsMorphController.GetMorphByUid("Right" + " " + morphName).morphValue;
                else
                    activeAtomsMorphController.GetMorphByUid("Right" + " " + morphName).morphValue = activeAtomsMorphController.GetMorphByUid("Left" + " " + morphName).morphValue;
            }
            public void ReverseMirror()
            {
                if (PHUI.GetHandSideText() == "Left")
                    activeAtomsMorphController.GetMorphByUid("Left" + " " + morphName).morphValue = activeAtomsMorphController.GetMorphByUid("Right" + " " + morphName).morphValue;
                else
                    activeAtomsMorphController.GetMorphByUid("Right" + " " + morphName).morphValue = activeAtomsMorphController.GetMorphByUid("Left" + " " + morphName).morphValue;
            }

            public void Swap()
            {
                float temp = activeAtomsMorphController.GetMorphByUid(PHUI.GetHandSideText() + " " + morphName).morphValue;
                activeAtomsMorphController.GetMorphByUid(PHUI.GetHandSideText() + " " + morphName).morphValue = activeAtomsMorphController.GetMorphByUid((PHUI.GetHandSideText() == "Right" ? "Left" : "Right") + " " + morphName).morphValue;
                activeAtomsMorphController.GetMorphByUid((PHUI.GetHandSideText() == "Right" ? "Left" : "Right") + " " + morphName).morphValue = temp;

            }

            void Update()
            {
                if (PHUI.isSymmetryToggled)
                {
                    Mirror();
                }
                if (!isMoused)
                {
                    UpdateSliderValue();
                    return;
                }
                if (Input.GetMouseButtonDown(PHUI.MOUSE_RIGHT))
                    ResetValue();
                if (Input.GetMouseButtonDown(PHUI.MOUSE_MIDDLE))
                {
                    if (PHInput.GetModifier(PHInput.MODIFIER_SHIFT))
                        Swap();
                    else if (PHInput.GetModifier(PHInput.MODIFIER_ALT))
                        Mirror();
                    else
                        ReverseMirror();
                }
                if (Input.GetMouseButton(PHUI.MOUSE_LEFT))
                {
                    if (PHUI.isSymmetryToggled)
                    {
                        activeAtomsMorphController.GetMorphByUid("Right" + " " + morphName).morphValue = slider.value;
                        activeAtomsMorphController.GetMorphByUid("Left" + " " + morphName).morphValue = slider.value;
                    }
                    else
                        activeAtomsMorphController.GetMorphByUid(PHUI.GetHandSideText() + " " + morphName).morphValue = slider.value;
                }
            }

        }
        public class JointSelector : MonoBehaviour, IPointerEnterHandler, IPointerExitHandler
        {
            public FingerJoint[] joints = new FingerJoint[2];

            public Image tex;
            string jointName;
            public string jointID;
            public int jointNumber;
            bool isMoused = false;

            public void SetName()
            {
                jointName = gameObject.name.Split('_')[2];
                jointID = jointName.Substring(0, jointName.Length - 1);
                jointNumber = jointName[jointName.Length - 1] - 48;

                tex = GetComponent<Image>();
            }
            public void OnPointerEnter(PointerEventData eventData)
            {
                isMoused = true;
            }
            public void OnPointerExit(PointerEventData eventData)
            {
                isMoused = false;
            }
            public void UpdateName(int carpal)
            {
                joints[0] = new FingerJoint('r', jointName, carpal);
                joints[1] = new FingerJoint('l', jointName, carpal);
            }
            public void Mirror()
            {
                Vector3 altRot = joints[PHUI.isRightSide ? 1 : 0].joint.baseJointRotation;
                joints[PHUI.isRightSide ? 0 : 1].SetRotation(new Vector3(altRot.x, -altRot.y, -altRot.z));
            }
            public void Swap()
            {
                Vector3 a = joints[PHUI.isRightSide ? 0 : 1].joint.baseJointRotation;
                joints[PHUI.isRightSide ? 0 : 1].SetRotation(joints[PHUI.isRightSide ? 1 : 0].joint.baseJointRotation);
                joints[PHUI.isRightSide ? 1 : 0].SetRotation(a);
            }
            public void ReverseMirror()
            {
                Vector3 altRot = joints[PHUI.isRightSide ? 0 : 1].joint.baseJointRotation;
                joints[PHUI.isRightSide ? 1 : 0].SetRotation(new Vector3(altRot.x, -altRot.y, -altRot.z));
            }
            void Update()
            {
                if (PHUI.isSymmetryToggled)
                {
                    ReverseMirror();
                }

                if (!isMoused)
                    return;

                if (Input.GetMouseButtonDown(PHUI.MOUSE_LEFT))
                {
                    Select();
                    UpdateRetrivedJoints();
                }

                if (Input.GetMouseButtonDown(PHUI.MOUSE_RIGHT))
                {
                    if (PHInput.GetModifier(PHInput.MODIFIER_CTRL))
                        foreach (var item in FindObjectsOfType<JointSelector>())
                            if (item.jointID == this.jointID && item.jointNumber > this.jointNumber)
                                item.joints[PHUI.isRightSide ? 0 : 1].Reset();
                    joints[PHUI.isRightSide ? 0 : 1].Reset();
                }

                if (Input.GetMouseButtonDown(PHUI.MOUSE_MIDDLE))
                {
                    if (PHInput.GetModifier(PHInput.MODIFIER_SHIFT))
                        Swap();
                    else if (PHInput.GetModifier(PHInput.MODIFIER_ALT))
                        ReverseMirror();
                    else
                        Mirror();
                }

            }
            public void Select()
            {
                if (!PHInput.GetModifier(PHInput.MODIFIER_CTRL))
                {
                    ClearActiveJoints();
                }
                if (PHInput.GetModifier(PHInput.MODIFIER_ALT))
                    if (this.jointID == "Carpal")
                        foreach (var item in FindObjectsOfType<JointSelector>().Where((js) => js.jointID == "Carpal"))
                        {
                            if (!activeJoints.Contains(item))
                            {
                                activeJoints.Add(item);
                                item.tex.color = Color.green;
                            }
                        }
                    else
                        foreach (var item in FindObjectsOfType<JointSelector>())
                            if (item.jointNumber == this.jointNumber && item.jointID != "Carpal" && !activeJoints.Contains(item))
                            {
                                activeJoints.Add(item);
                                item.tex.color = Color.green;
                            }
                if (PHInput.GetModifier(PHInput.MODIFIER_SHIFT))
                    foreach (var item in FindObjectsOfType<JointSelector>())
                        if (item.jointID == this.jointID && item.jointNumber > this.jointNumber && !activeJoints.Contains(item))
                        {
                            activeJoints.Add(item);
                            item.tex.color = Color.green;
                        }
                if (!activeJoints.Contains(this))
                {
                    activeJoints.Add(this);
                    tex.color = Color.green;
                }
            }
            public void Clear()
            {
                tex.color = Color.white;
            }
        }
        public static void RotateActiveHandBones(Vector3 dir)
        {
            PHUtils.activeAtom.GetStorableByID("RightHandControl").SetStringChooserParamValue("fingerControlMode", "Morphs");
            PHUtils.activeAtom.GetStorableByID("LeftHandControl").SetStringChooserParamValue("fingerControlMode", "Morphs");
            dir *= PH.sensetivity.valNoCallback;
            foreach (var joint in activeJoints)
            {
                joint.joints[PHUI.isRightSide ? 0 : 1].Rotate(dir);
            }
        }
        public class PHHandSideButton : MonoBehaviour, IPointerClickHandler
        {
            public void OnPointerClick(PointerEventData eventData)
            {
                if (eventData.button == PointerEventData.InputButton.Left)
                    SwitchSides();
                else if (eventData.button == PointerEventData.InputButton.Middle)
                {
                    if (ActiveAtomHasPlugin())
                        foreach (var item in GameObject.FindObjectsOfType<PHHands.JointSelector>())
                            item.ReverseMirror();
                    foreach (var item in GameObject.FindObjectsOfType<PHHands.PHSlider>())
                        item.ReverseMirror();
                }
            }
        }
    }
}
