﻿using Leap.Unity;
using System;
using System.Linq;
using System.Runtime.Serialization;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;

namespace Ruvik
{
    class PHMove
    {
        static GameObject PHMoveUI;
        static Button globalBtn,localBtn,nodeBtn,camBtn;
        static Color enabledColor = Color.white, disabledColor = Color.gray;
        public static bool isLocal = true;
        public static float rotationValue = 10.0f, positionValue = 0.1f, rotatoinIncrements = 15.0f;
        const int AXIS_X = 0;
        const int AXIS_Y = 1;
        const int AXIS_Z = 2;
        const int GLOBAL = 0;
        const int LOCAL = 1;
        const int NODE = 2;
        const int CAM = 3;


        static void DisableButton(Button btn)
        {
            btn.image.color = disabledColor;
        }
        static void EnableButton(Button btn)
        {
            btn.image.color = enabledColor;
        }
        public static void InitUI()
        {
            PHMoveUI = GameObject.Find("PH_movement_menu");
            GameObject.Find("PH_movement_alignments").gameObject.AddComponent<PHMoveAlignmentControl>();
            GameObject.Find("PH_movement_position").gameObject.AddComponent<PHMovePosition>();
            GameObject.Find("PH_movement_rotation").gameObject.AddComponent<PHMoveRotation>();
            InitControllerUI();
            PHMoveUI.SetActive(false);
        }
        public static void ToggleUI()
        {
            PHMoveUI.SetActive(!PHMoveUI.activeInHierarchy);
        }
        public static void InitControllerUI()
        {
            GameObject.Find("PH_movement_rotation_control").AddComponent<PHMoveRotationControl>();
            GameObject.Find("PH_movement_position_control").AddComponent<PHMovePositionControl>();
        }
        public static void ResetRotation(Node node)
        {
            PHUndo.StoreState();
            if (node.controller.name == "control")
            {
                node.controller.RotateControl(Vector3.zero);
                return;
            }
            Vector3 defaultRot = PHUtils.activeAtom.mainController.transform.eulerAngles + node.controller.startingLocalRotation.eulerAngles;
            node.controller.RotateControl(defaultRot);

        }
        public static void ResetPosition(Node node)
        {
            PHUndo.StoreState();
            if (node.controller.name == "control")
            {
                node.controller.MoveTo(Vector3.zero);
                return;
            }
            Vector3 defaultPos =
                PHUtils.activeAtom.mainController.transform.position +
                (PHUtils.activeAtom.mainController.transform.right * node.controller.startingLocalPosition.x) +
                (PHUtils.activeAtom.mainController.transform.up * node.controller.startingLocalPosition.y) +
                (PHUtils.activeAtom.mainController.transform.forward * node.controller.startingLocalPosition.z);
            node.controller.MoveControl(defaultPos);
        }
        public static void ResetRotationAxis(Node node, Vector3 vec, Vector3 invVec)
        {
            PHUndo.StoreState();
            if (node.controller.name == "control")
            {
                node.controller.RotateControl(Vector3.Scale(node.controller.transform.eulerAngles, invVec));
                return;
            }
            Vector3 defaultRot = PHUtils.activeAtom.mainController.transform.eulerAngles + node.controller.startingLocalRotation.eulerAngles;
            node.controller.RotateControl(Vector3.Scale(defaultRot, vec) + Vector3.Scale(node.controller.transform.eulerAngles, invVec));
        }
        static Vector3 GetAxisInverse(int axis)
        {
            switch (axis)
            {
                case AXIS_X:
                    return new Vector3(0, 1, 1);
                case AXIS_Y:
                    return new Vector3(1, 0, 1);
                default:
                    return new Vector3(1, 1, 0);
            }
        }
        static Vector3 GetAxisVector(int axis)
        {
            switch (axis)
            {
                case AXIS_X:
                    return new Vector3(1, 0, 0);
                case AXIS_Y:
                    return new Vector3(0, 1, 0);
                default:
                    return new Vector3(0, 0, 1);
            }
        }
 
        public static void ResetPositionAxis(Node node, int axis)
        {
            PHUndo.StoreState();
            Vector3 vec = Vector3.zero;
            Vector3 local = node.controller.GetLocalPosition();
            Vector3 starting = node.controller.startingPosition;
            switch (axis)
            {
                case AXIS_X:
                    vec = new Vector3(starting.x, local.y, local.z);
                    break;
                case AXIS_Y:
                    vec = new Vector3(local.x, starting.y, local.z);
                    break;
                case AXIS_Z:
                    vec = new Vector3(local.x, local.y, starting.z);
                    break;
            }
            if (node.controller.name == "control")
            {
                node.controller.MoveControl(vec);
                return;
            }
            Vector3 defaultPos =
                PHUtils.activeAtom.mainController.transform.position +
                (PHUtils.activeAtom.mainController.transform.right * vec.x) +
                (PHUtils.activeAtom.mainController.transform.up * vec.y) +
                (PHUtils.activeAtom.mainController.transform.forward * vec.z);
            node.controller.MoveControl(defaultPos);
        }

        delegate void NodeMovementMethod(int axis,float i);
        delegate void NodeRotationMethod(int axis,float i);

        delegate void NodeRelativeMovementMethod(Vector3 dir);
        delegate void NodeRelativeRotationMethod(Vector3 dir);

        static NodeMovementMethod MoveController;
        static NodeRotationMethod RotateController;

        static NodeRelativeMovementMethod MoveControllerRelative;
        static NodeRelativeRotationMethod RotateControllerRelative;
        public static string[] GetNodePosition(FreeControllerV3 node)
        {
            switch (PHMoveAlignmentControl.currentAlignment)
            {
                case GLOBAL:
                    Vector3 globalPos = node.transform.position;
                    return new string[] { $"{Math.Round(globalPos.x, 4)}",
                                $"{Math.Round(globalPos.y, 4)}",
                                $"{Math.Round(globalPos.z, 4)}" };
                case LOCAL:
                    Vector3 localPos = node.GetLocalPosition();
                    return new string[] { $"{Math.Round(localPos.x, 4)}",
                                $"{Math.Round(localPos.y, 4)}",
                                $"{Math.Round(localPos.z, 4)}" };
                case NODE:
                    Vector3 nodePos = node.GetLocalPosition() - node.startingLocalPosition;
                    return new string[] {
                        $"{Math.Round(nodePos.x, 4)}",
                                $"{Math.Round(nodePos.y, 4)}",
                                $"{Math.Round(nodePos.z, 4)}"
                    };
                default:
                    return new string[] { "", "", "" };
            }
        }
        public static string[] GetNodeRotation(FreeControllerV3 node)
        {
            switch (PHMoveAlignmentControl.currentAlignment)
            {
                case GLOBAL:
                    Vector3 globalRot = node.transform.eulerAngles;
                    return new string[] { $"{Math.Round(globalRot.x, 4)}",
                                $"{Math.Round(globalRot.y, 4)}",
                                $"{Math.Round(globalRot.z, 4)}" };
                case LOCAL:
                    Vector3 localRot = node.GetLocalEulerAngles();
                    return new string[] { $"{Math.Round(localRot.x, 4)}",
                                $"{Math.Round(localRot.y, 4)}",
                                $"{Math.Round(localRot.z, 4)}" };
                case NODE:
                    Vector3 nodeRot = node.GetLocalEulerAngles() - node.startingLocalRotation.eulerAngles;
                    return new string[] { $"{Math.Round(nodeRot.x, 4)}",
                                $"{Math.Round(nodeRot.y, 4)}",
                                $"{Math.Round(nodeRot.z, 4)}" };
                default:
                    return new string[] { "", "", "" };
            }
        }

        public static void MoveGlobalRelative(Vector3 dir)
        {
            foreach (var node in PHUtils.activeNodes)
            {
                node.controller.MoveControlRelatve(dir);
                if (PHUI.isSymmetryToggled && node.altNode != null)
                    node.altNode.controller.MoveControlRelatve(dir);
            }
        }
        public static void MoveLocalRelative(Vector3 dir)
        {
            foreach (var node in PHUtils.activeNodes)
            {
                if (node.controller.name == "control")
                {
                    node.controller.MoveControlRelatve(dir);
                    continue;
                }
                Vector3 vec = dir + node.controller.GetLocalPosition();
                Vector3 moveTo =
                    PHUtils.activeAtom.mainController.transform.position +
                    (PHUtils.activeAtom.mainController.transform.right * vec.x) +
                    (PHUtils.activeAtom.mainController.transform.up * vec.y) +
                    (PHUtils.activeAtom.mainController.transform.forward * vec.z);
                node.controller.MoveControl(moveTo);
                if (PHUI.isSymmetryToggled && node.altNode != null)
                {

                    vec = dir + node.altNode.controller.GetLocalPosition();
                    moveTo =
                        PHUtils.activeAtom.mainController.transform.position +
                        (PHUtils.activeAtom.mainController.transform.right * vec.x) +
                        (PHUtils.activeAtom.mainController.transform.up * vec.y) +
                        (PHUtils.activeAtom.mainController.transform.forward * vec.z);
                    node.altNode.controller.MoveControl(moveTo);
                }
            }
        }
        public static void MoveNodeRelative(Vector3 dir)
        {
            foreach (var node in PHUtils.activeNodes)
            {
                Vector3 moveTo =
                    (node.controller.transform.right * dir.x) +
                    (node.controller.transform.up * dir.y) +
                    (node.controller.transform.forward * dir.z);
                node.controller.MoveControlRelatve(moveTo);
                if (PHUI.isSymmetryToggled && node.altNode != null)
                {
                    moveTo =
                        (node.altNode.controller.transform.right * dir.x) +
                        (node.altNode.controller.transform.up * dir.y) +
                        (node.altNode.controller.transform.forward * dir.z);
                    node.altNode.controller.MoveControlRelatve(moveTo);
                }
            }
        }
        public static void MoveCamRelative(Vector3 dir)
        {
            foreach (var node in PHUtils.activeNodes)
            {
                Vector3 move =
                    SuperController.singleton.lookCamera.transform.right * dir.x +
                    SuperController.singleton.lookCamera.transform.up * dir.y +
                    SuperController.singleton.lookCamera.transform.forward * dir.z
                    ;
                node.controller.MoveControlRelatve(move);
                if (PHUI.isSymmetryToggled && node.altNode != null)
                {
                    node.altNode.controller.MoveControlRelatve(move);
                }
            }
        }
        public static void MoveGlobal(int axis, float i)
        {
            foreach (var node in PHUtils.activeNodes)
            {
                Vector3 a = Vector3.Scale(node.controller.transform.position, GetAxisInverse(axis));
                Vector3 b = a + (GetAxisVector(axis) * i);
                node.controller.MoveControl(b);
            }

        }
        public static void MoveLocal(int axis, float i)
        {
            var mainController = PHUtils.activeAtom.mainController.transform;
            foreach (var node in PHUtils.activeNodes)
                if (node.controller.name == "control")
                {
                    Vector3 a = Vector3.Scale(mainController.position, GetAxisInverse(axis));
                    Vector3 b = GetAxisVector(axis) * i;
                    node.controller.MoveControl(a + b);
                }
                else
                {
                    Vector3 a = Vector3.Scale(node.controller.GetLocalPosition(), GetAxisInverse(axis));
                    Vector3 b = a + (GetAxisVector(axis) * i);

                    node.controller.MoveControl(
                        mainController.position +
                        (mainController.right * b.x) +
                        (mainController.up * b.y) +
                        (mainController.forward * b.z)
                        );
                }
        }
        public static void MoveNode(int axis, float i)
        {
            foreach (var node in PHUtils.activeNodes)
            {
                Vector3 a = Vector3.Scale(Vector3.one * i, GetAxisVector(axis));
                Vector3 b =
                    node.controller.transform.right * a.x +
                    node.controller.transform.up * a.y +
                    node.controller.transform.forward * a.z;
                node.controller.MoveControl(node.controller.transform.position + b);
                if (PHUI.isSymmetryToggled && node.altNode != null)
                {
                    b =
                            node.altNode.controller.transform.right * a.x +
                            node.altNode.controller.transform.up * a.y +
                            node.altNode.controller.transform.forward * a.z;
                    node.altNode.controller.MoveControl(node.altNode.controller.transform.position + b);
                }
            }
        }
        public static void MoveCam(int axis, float i)
        {
            SuperController.LogMessage("DON'T DO THIS");
        }
        public static void RotateGlobalRelative(Vector3 dir)
        {
            foreach (var node in PHUtils.activeNodes)
            {
                node.controller.RotateControl(node.controller.transform.rotation.eulerAngles + dir);
                if (PHUI.isSymmetryToggled && node.altNode != null)
                    node.altNode.controller.RotateControl(node.altNode.controller.transform.rotation.eulerAngles + dir);
            }
        }
        public static void RotateLocalRelative(Vector3 dir)
        {
            foreach (var node in PHUtils.activeNodes)
            {
                node.controller.RotateControl(node.controller.transform.rotation.eulerAngles + dir);
                if (PHUI.isSymmetryToggled && node.altNode != null)
                    node.altNode.controller.RotateControl(node.altNode.controller.transform.rotation.eulerAngles + dir);
            }
        }
        public static void RotateNodeRelative(Vector3 dir)
        {
            foreach (var node in PHUtils.activeNodes)
            {
                node.controller.RotateControl(node.controller.transform.rotation.eulerAngles + dir);
                if (PHUI.isSymmetryToggled && node.altNode != null)
                    node.altNode.controller.RotateControl(node.altNode.controller.transform.rotation.eulerAngles + dir);
            }
        }
        public static void RotateCamRelative(Vector3 dir)
        {
            foreach (var node in PHUtils.activeNodes)
            {
                node.controller.RotateControl(node.controller.transform.rotation.eulerAngles + dir);
                if (PHUI.isSymmetryToggled && node.altNode != null)
                    node.altNode.controller.RotateControl(node.altNode.controller.transform.rotation.eulerAngles + dir);
            }
        }
        public static void RotateGlobal(int axis, float i)
        {
            foreach (var node in PHUtils.activeNodes)
            {
                Vector3 a = Vector3.Scale(node.controller.transform.eulerAngles, GetAxisInverse(axis));
                Vector3 b = GetAxisVector(axis) * i;
                node.controller.RotateControl(a + b);
            }
        }
        public static void RotateLocal(int axis, float i)
        {
            foreach (var node in PHUtils.activeNodes)
            {
                Vector3 a = Vector3.Scale(node.controller.transform.eulerAngles, GetAxisInverse(axis));
                Vector3 b = Vector3.Scale(PHUtils.activeAtom.mainController.GetLocalEulerAngles() + (GetAxisVector(axis) * i), GetAxisVector(axis));
                node.controller.RotateControl(a + b);
            }
        }
        public static void RotateNode(int axis, float i)
        {
            foreach (var node in PHUtils.activeNodes)
            {
                Vector3 a = Vector3.Scale(Vector3.one * i, GetAxisVector(axis));
                Vector3 b = Vector3.Scale(node.controller.transform.eulerAngles, GetAxisInverse(axis));
                node.controller.RotateControl(a+b);
                if (PHUI.isSymmetryToggled && node.altNode != null)
                {
                    b = Vector3.Scale(node.altNode.controller.transform.eulerAngles, GetAxisInverse(axis));
                    node.altNode.controller.MoveControl(a + b);
                }
            }
        }
        public static void RotateCam(int axis, float i)
        {
            //DONT
        }
        class PHMoveAlignmentControl : MonoBehaviour
        {
            public static int currentAlignment = 0;
            void Awake()
            {
                foreach (var button in this.transform.GetChildren())
                {
                    switch (button.name.Split('_')[2])
                    {
                        case "local":
                            localBtn = button.GetComponent<Button>();
                            break;
                        case "global":
                            globalBtn = button.GetComponent<Button>();
                            break;
                        case "node":
                            nodeBtn = button.GetComponent<Button>();
                            break;
                        case "cam":
                            camBtn = button.GetComponent<Button>();
                            break;
                    }
                }

                MoveController = MoveGlobal;
                MoveControllerRelative = MoveGlobalRelative;
                RotateController = RotateGlobal;
                RotateControllerRelative = RotateGlobalRelative;
                EnableButton(globalBtn);
                DisableButton(localBtn);
                DisableButton(nodeBtn);
                DisableButton(camBtn);

                globalBtn.onClick.AddListener(() =>
                {
                    PHMovePosition.SetInputInteractable(true);
                    PHMoveRotation.SetInputInteractable(true);
                    currentAlignment = GLOBAL;
                    MoveController = MoveGlobal;
                    MoveControllerRelative = MoveGlobalRelative;
                    RotateController = RotateGlobal;
                    RotateControllerRelative = RotateGlobalRelative;
                    EnableButton(globalBtn);
                    DisableButton(localBtn);
                    DisableButton(nodeBtn);
                    DisableButton(camBtn);
                });
                localBtn.onClick.AddListener(() =>
                {
                    PHMovePosition.SetInputInteractable(true);
                    PHMoveRotation.SetInputInteractable(true);
                    currentAlignment = LOCAL;
                    MoveController = MoveLocal;
                    MoveControllerRelative = MoveLocalRelative;
                    RotateController = RotateLocal;
                    RotateControllerRelative = RotateLocalRelative;
                    DisableButton(globalBtn);
                    EnableButton(localBtn);
                    DisableButton(nodeBtn);
                    DisableButton(camBtn);
                });
                nodeBtn.onClick.AddListener(() =>
                {
                    PHMovePosition.SetInputInteractable(false);
                    PHMoveRotation.SetInputInteractable(false);
                    currentAlignment = NODE;
                    MoveController = MoveNode;
                    MoveControllerRelative = MoveNodeRelative;
                    RotateController = RotateNode;
                    RotateControllerRelative = RotateNodeRelative;
                    DisableButton(globalBtn);
                    DisableButton(localBtn);
                    EnableButton(nodeBtn);
                    DisableButton(camBtn);
                });
                camBtn.onClick.AddListener(() =>
                {
                    PHMovePosition.SetInputInteractable(false);
                    PHMoveRotation.SetInputInteractable(false);
                    currentAlignment = CAM;
                    MoveController = MoveCam;
                    MoveControllerRelative = MoveCamRelative;
                    RotateController = RotateCam;
                    RotateControllerRelative = RotateCamRelative;
                    DisableButton(localBtn);
                    DisableButton(globalBtn);
                    DisableButton(nodeBtn);
                    EnableButton(camBtn);
                });
            }
        }
        class PHMoveRotationControl : MonoBehaviour
        {
            InputField input;
            float valueMin = 1.0f, valueMax = 90.0f;
            void Awake()
            {
                input = transform.Find("PH_movement_value").gameObject.GetComponent<InputField>();
                transform.Find("PH_movement_dec").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    rotationValue -= rotatoinIncrements;
                    rotationValue = Math.Max(valueMin, Math.Min(valueMax, rotationValue));
                });
                transform.Find("PH_movement_inc").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    rotationValue += rotatoinIncrements;
                    rotationValue = Math.Max(valueMin, Math.Min(valueMax, rotationValue));
                });
                input.onEndEdit.AddListener((val) =>
                {
                    rotationValue = (float)Convert.ToDouble(val.Replace(",", "."));
                    rotationValue = Math.Max(valueMin, Math.Min(valueMax, rotationValue));
                });
            }
            void Update()
            {
                if (input.isFocused)
                    return;
                input.text = PHMove.rotationValue.ToString();
            }
        }
        class PHMovePositionControl : MonoBehaviour
        {
            InputField input;
            float valueMin = 0.0001f, valueMax = 1.0f, multiBy = 10.0f;
            void Awake()
            {
                input = transform.Find("PH_movement_value").gameObject.GetComponent<InputField>();
                transform.Find("PH_movement_dec").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    float newVal = positionValue / multiBy;
                    if (newVal > valueMin && newVal < valueMax)
                        positionValue = newVal;
                });
                transform.Find("PH_movement_inc").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    float newVal = positionValue * multiBy;
                    if (newVal > valueMin && newVal < valueMax)
                        positionValue = newVal;
                });
                input.onEndEdit.AddListener((val) =>
                {
                    float newVal = (float)Convert.ToDouble(val.Replace(",", "."));
                    //DANGER!!!
                    while (newVal < valueMin) newVal *= multiBy;
                    while (newVal > valueMax) newVal /= multiBy;
                    positionValue = newVal;
                });
            }
            void Update()
            {
                if (input.isFocused)
                    return;
                input.text = Math.Round(positionValue, 5).ToString();
            }
        }
        class RightClickToResetRotation : MonoBehaviour, IPointerClickHandler
        {
            Vector3 vec;
            Vector3 invVec;
            public void Awake()
            {
                switch (transform.parent.name.Split('_').Last())
                {
                    case "x":
                        vec = Vector3.right;
                        invVec = new Vector3(0, 1, 1);
                        break;
                    case "y":
                        vec = Vector3.up;
                        invVec = new Vector3(1, 0, 1);
                        break;
                    case "z":
                        vec = Vector3.forward;
                        invVec = new Vector3(1, 1, 0);
                        break;
                }
            }
            public void OnPointerClick(PointerEventData eventData)
            {
                if (eventData.button == PointerEventData.InputButton.Left || eventData.button == PointerEventData.InputButton.Middle)
                    return;
                PHUndo.StoreState();
                foreach (var node in PHUtils.activeNodes)
                {
                    ResetRotationAxis(node, vec, invVec);
                    if (PHUI.isSymmetryToggled && node.altNode != null)
                        ResetRotationAxis(node.altNode, vec, invVec);
                }
            }
        }
        class RightClickToResetPosition : MonoBehaviour, IPointerClickHandler
        {
            int axis;
            public void Awake()
            {
                switch (transform.parent.name.Split('_').Last())
                {
                    case "x":
                        axis = AXIS_X;
                        break;
                    case "y":
                        axis = AXIS_Y;
                        break;
                    case "z":
                        axis = AXIS_Z;
                        break;
                }
            }
            public void OnPointerClick(PointerEventData eventData)
            {
                if (eventData.button == PointerEventData.InputButton.Left || eventData.button == PointerEventData.InputButton.Middle)
                    return;
                PHUndo.StoreState();
                foreach (var node in PHUtils.activeNodes)
                {
                    ResetPositionAxis(node, axis);
                    if (PHUI.isSymmetryToggled && node.altNode != null)
                        ResetPositionAxis(node.altNode, axis);
                }
            }
        }
        class PHMovePosition : MonoBehaviour
        {
            static InputField x, y, z;
            void InitAxis()
            {
                transform.Find("PH_movement_x").Find("PH_movement_inc").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    PHUndo.StoreState();
                    MoveControllerRelative(new Vector3(positionValue, 0, 0));
                });
                transform.Find("PH_movement_x").Find("PH_movement_dec").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    PHUndo.StoreState();
                    MoveControllerRelative(new Vector3(-positionValue, 0, 0));
                });
                transform.Find("PH_movement_y").Find("PH_movement_inc").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    PHUndo.StoreState();
                    MoveControllerRelative(new Vector3(0, positionValue, 0));
                });
                transform.Find("PH_movement_y").Find("PH_movement_dec").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    PHUndo.StoreState();
                    MoveControllerRelative(new Vector3(0, -positionValue, 0));
                });
                transform.Find("PH_movement_z").Find("PH_movement_inc").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    PHUndo.StoreState();
                    MoveControllerRelative(new Vector3(0, 0, positionValue));
                });
                transform.Find("PH_movement_z").Find("PH_movement_dec").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    PHUndo.StoreState();
                    MoveControllerRelative(new Vector3(0, 0, -positionValue));
                });
            }
            void Awake()
            {
                x = transform.Find("PH_movement_x").GetComponentInChildren<InputField>();
                y = transform.Find("PH_movement_y").GetComponentInChildren<InputField>();
                z = transform.Find("PH_movement_z").GetComponentInChildren<InputField>();
                InitAxis();

                transform.Find("PH_movement_reset").GetComponent<UnityEngine.UI.Button>().onClick.AddListener(() =>
                {
                    PHUndo.StoreState();
                    foreach (var node in PHUtils.activeNodes)
                    {
                        ResetPosition(node);
                        if (PHUI.isSymmetryToggled && node.altNode != null)
                            ResetPosition(node.altNode);
                    }
                });
                x.gameObject.AddComponent<RightClickToResetPosition>();
                x.onEndEdit.AddListener((str) =>
                {
                    if (PHUtils.activeAtom == null)
                        return;
                    PHUndo.StoreState();
                    float i = (float)Convert.ToDouble(str.Replace(',', '.'));
                    MoveController(AXIS_X, i);
                });
                y.gameObject.AddComponent<RightClickToResetPosition>();
                y.onEndEdit.AddListener((str) =>
                {
                    if (PHUtils.activeAtom == null)
                        return;
                    PHUndo.StoreState();
                    Transform mainController = PHUtils.activeAtom.mainController.transform;
                    float i = (float)Convert.ToDouble(str.Replace(',', '.'));
                    MoveController(AXIS_Y, i);
                });
                z.gameObject.AddComponent<RightClickToResetPosition>();
                z.onEndEdit.AddListener((str) =>
                {
                    if (PHUtils.activeAtom == null)
                        return;
                    PHUndo.StoreState();
                    float i = (float)Convert.ToDouble(str.Replace(',', '.'));
                    Transform mainController = PHUtils.activeAtom.mainController.transform;
                    MoveController(AXIS_Z, i);
                });
            }
            void Update()
            {
                switch (PHUtils.activeNodes.Count)
                {
                    case 0:
                        x.text = "";
                        y.text = "";
                        z.text = "";
                        break;
                    case 1:
                        var controller = PHUtils.activeNodes.First().controller;
                        string[] axis = GetNodePosition(controller);
                        if (!x.isFocused)
                            x.text = axis[0];
                        if (!y.isFocused)
                            y.text = axis[1];
                        if (!z.isFocused)
                            z.text = axis[2];
                        break;
                    default:
                        if (!x.isFocused)
                            x.text = "...";
                        if (!y.isFocused)
                            y.text = "...";
                        if (!z.isFocused)
                            z.text = "...";
                        break;

                }
            }
            public static void SetInputInteractable(bool on)
            {
                x.interactable = on;
                y.interactable = on;
                z.interactable = on;
            }
        }
        class PHMoveRotation : MonoBehaviour
        {
            static InputField x, y, z;
            void InitAxis()
            {
                transform.Find("PH_movement_x").Find("PH_movement_inc").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    PHUndo.StoreState();
                    RotateControllerRelative(new Vector3(rotationValue, 0, 0));
                });
                transform.Find("PH_movement_x").Find("PH_movement_dec").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    PHUndo.StoreState();
                    RotateControllerRelative(new Vector3(-rotationValue, 0, 0));
                });
                transform.Find("PH_movement_y").Find("PH_movement_inc").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    PHUndo.StoreState();
                    RotateControllerRelative(new Vector3(0, rotationValue, 0));
                });
                transform.Find("PH_movement_y").Find("PH_movement_dec").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    PHUndo.StoreState();
                    RotateControllerRelative(new Vector3(0, -rotationValue, 0));
                });
                transform.Find("PH_movement_z").Find("PH_movement_inc").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    PHUndo.StoreState();
                    RotateControllerRelative(new Vector3(0, 0, rotationValue));
                });
                transform.Find("PH_movement_z").Find("PH_movement_dec").gameObject.GetComponent<Button>().onClick.AddListener(() =>
                {
                    PHUndo.StoreState();
                    RotateControllerRelative(new Vector3(0, 0, -rotationValue));
                });
            }

            void Awake()
            {
                x = transform.Find("PH_movement_x").GetComponentInChildren<InputField>();
                y = transform.Find("PH_movement_y").GetComponentInChildren<InputField>();
                z = transform.Find("PH_movement_z").GetComponentInChildren<InputField>();
                InitAxis();

                transform.Find("PH_movement_reset").GetComponent<UnityEngine.UI.Button>().onClick.AddListener(() =>
                {
                    PHUndo.StoreState();
                    foreach (var node in PHUtils.activeNodes)
                    {
                        ResetRotation(node);
                        if (PHUI.isSymmetryToggled && node.altNode == null)
                            ResetRotation(node.altNode);
                    }
                });
                x.gameObject.AddComponent<RightClickToResetRotation>();
                x.onEndEdit.AddListener((str) =>
                {
                    if (PHUtils.activeAtom == null)
                        return;
                    PHUndo.StoreState();
                    float i = (float)Convert.ToDouble(str.Replace(',', '.'));
                    RotateController(AXIS_X, i);
                });
                y.gameObject.AddComponent<RightClickToResetRotation>();
                y.onEndEdit.AddListener((str) =>
                {
                    if (PHUtils.activeAtom == null)
                        return;
                    PHUndo.StoreState();
                    float i = (float)Convert.ToDouble(str.Replace(',', '.'));
                    RotateController(AXIS_Y, i);
                });
                z.gameObject.AddComponent<RightClickToResetRotation>();
                z.onEndEdit.AddListener((str) =>
                {
                    if (PHUtils.activeAtom == null)
                        return;
                    PHUndo.StoreState();
                    float i = (float)Convert.ToDouble(str.Replace(',', '.'));
                    RotateController(AXIS_Z, i);
                });
            }
            void Update()
            {
                switch (PHUtils.activeNodes.Count)
                {
                    case 0:
                        x.text = "";
                        y.text = "";
                        z.text = "";
                        break;
                    case 1:
                        var controller = PHUtils.activeNodes.First().controller;
                        string[] axis = GetNodeRotation(controller);
                        if (!x.isFocused)
                            x.text = axis[0];
                        if (!y.isFocused)
                            y.text = axis[1];
                        if (!z.isFocused)
                            z.text = axis[2];
                        break;
                    default:
                        if (!x.isFocused)
                            x.text = "...";
                        if (!y.isFocused)
                            y.text = "...";
                        if (!z.isFocused)
                            z.text = "...";
                        break;

                }
            }
            public static void SetInputInteractable(bool on)
            {
                x.interactable = on;
                y.interactable = on;
                z.interactable = on;
            }
        }
    }
}
